import { database } from "./database";
import { BasicEntity, DateTimeColumn, IntColumn, JoinOne, StringColumn, Table } from "./entity";

database.config(() => ({
	host: "127.0.0.1",
	port: 5432,
	user: "postgres",
	password: "ljk5408828",
	database: "urnas",
	max: 100,
}));


/** 数据实体 */
class DataEntity extends BasicEntity {
	/** ID */
	@IntColumn({ primary: true }) declare id: number
}

/** 带有时间的实体 */
class TimedEntity extends DataEntity {
	/** 创建时间 */
	@DateTimeColumn() declare ctime: string

	/** 修改时间 */
	@DateTimeColumn() declare utime: string | null

	/** 删除时间 */
	@DateTimeColumn() declare dtime: string | null
}


export enum UserRole {
	/** 系统管理员 */
	SystemAdmin = 1 << 0,
	/** 密码本管理员*/
	CodeBookAdmin = 1 << 1,
	/** 相册管理员 */
	AlbumAdmin = 1 << 2,
	/** FM管理员 */
	FMAdmin = 1 << 3,

	/** 超级用户 */
	Super = SystemAdmin | CodeBookAdmin | AlbumAdmin | FMAdmin,
	/** 常规用户 */
	User = 0,
};

/** 用户 */
@Table("user.user")
export class User extends TimedEntity {
	/** 账号 */
	@StringColumn() declare number: string

	/** 密码 */
	@StringColumn() declare password: string

	/** 昵称 */
	@StringColumn() declare name: string

	/** 角色 */
	@IntColumn() declare role: UserRole

	/** 邮箱 */
	@StringColumn() declare email: string | null

	/** 手机 */
	@StringColumn() declare phone: string | null

	/** 头像 */
	@StringColumn() declare avatar: string | null

	/** 登录过期时间 */
	@DateTimeColumn() declare ttime: string
}

@Table("user.user_login")
export class UserLogin extends DataEntity {
	@IntColumn() declare userID: number

	@JoinOne(() => User, "userID") declare user: User

	@JoinOne(() => User, "userID") declare test: User

	@StringColumn() declare device: string | null

	@StringColumn() declare ip: string

	@DateTimeColumn() declare ctime: string
}

database.select(UserLogin)
	.join("user")
	.join("test")
	.find()
	.then(res => {
		console.log(res);
	}).catch(err => {
		console.error(err);
	})
	.finally(() => {
		console.log("END")
		database.close();
	});